#include <QtGui>

#include "mkvextractgui.h"

mkvextractgui::mkvextractgui(QWidget *parent): QDialog(parent)
{
	setupUi(this);

	i=0;

	QTextCodec::setCodecForTr(QTextCodec::codecForName("UTF-8"));

	QTimer::singleShot(0, this, SLOT(mkvextracttest()));
	QTimer::singleShot(0, this, SLOT(mediainfotest()));

	// Verhindert die direkte Eingabe von Werten in der SpinBox
//	HQspinBox->findChild<QLineEdit*>()->setReadOnly(true);

	connect(extractOpenButton, SIGNAL(clicked()), this, SLOT(extractOpen()));
	connect(extractSaveButton, SIGNAL(clicked()), this, SLOT(extractSave()));
	connect(extractStartButton, SIGNAL(clicked()), this, SLOT(extractStart()));

	connect(&extractProc, SIGNAL(readyReadStandardOutput()), this, SLOT(extractFortschritt()));
	connect(&extractProc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(extractProcBeendet(int, QProcess::ExitStatus)));

	connect(&extractInfo, SIGNAL(readyReadStandardOutput()), this, SLOT(extractInfoFortschritt()));
	connect(&extractInfo, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(extractInfoBeendet()));

}



/********************************************************************************************/
/*                                   mkvextract                                             */
/********************************************************************************************/


// Öffnen des extract-Files und Eintragen der einzelnen Spuren in das QTableWidget für extract

void mkvextractgui::extractOpen()
{
	QString initialDir = extractOpenLineEdit->text();
	if(initialDir.isEmpty())
		initialDir = QDir::homePath();

	QString InpFile = QFileDialog::getOpenFileName(
		this,
		"Open File Dialog",
		initialDir,
		"mkv-File(*.mkv);;All Files(*.*)");

	if(!InpFile.isEmpty())
		{
		QFileInfo file(InpFile);
		QString FilePath = InpFile;
		FilePath.remove(file.fileName());

		extractOpenLineEdit->setText(InpFile);
		QString saveDir=extractSaveLineEdit->text();
		if(saveDir.isEmpty())
			extractSaveLineEdit->setText(FilePath);

		extractTableWidget->clear();
		extractTableWidget->setRowCount(0);
		extractTableWidget->setColumnCount(4);
		extractTableWidget->horizontalHeader()->resizeSection(0,30);
		extractTableWidget->horizontalHeader()->resizeSection(1,460);
		extractTableWidget->horizontalHeader()->resizeSection(2,80);
		extractTableWidget->horizontalHeader()->resizeSection(3,50);
		extractTableWidget->setHorizontalHeaderItem(0, new QTableWidgetItem("Sel"));
		extractTableWidget->setHorizontalHeaderItem(1, new QTableWidgetItem("Track"));
		extractTableWidget->setHorizontalHeaderItem(2, new QTableWidgetItem("Type"));
		extractTableWidget->setHorizontalHeaderItem(3, new QTableWidgetItem("Suffix"));

		extractProgressBar->setValue(0);
		i=0;

		extractInfoStart();
		}


}

// Speicherordner für extract auswählen

void mkvextractgui::extractSave()
{
	QString initialDir = extractSaveLineEdit->text();
	if(initialDir.isEmpty())
		initialDir = QDir::homePath();

	QString SaveDir = saveDialog(initialDir);

	if(!SaveDir.isEmpty())
		extractSaveLineEdit->setText(SaveDir);
}

// extractions-Vorgang

void mkvextractgui::extractStart()
{
	if(extractStartButton->text()=="Extract")
		{
		QString SourceFile;
		QString TargetDirectory;
		QString BaseFile;
	
		extractStartButton->setText("Stop");
		extractOpenButton->setEnabled(false);
		extractSaveButton->setEnabled(false);
	
		SourceFile = extractOpenLineEdit->text();
		TargetDirectory = extractSaveLineEdit->text();
		BaseFile = TargetDirectory+QFileInfo(SourceFile).completeBaseName();

		jobs = extractTableWidget->rowCount();
		QStringList arg;
		arg << "tracks" << SourceFile;

		QList<QCheckBox *> current_CheckBox = extractTableWidget->findChildren<QCheckBox *>();

		int x;
		for(x=0;x<jobs;x++)
			{
			QString current_Track;
			QString Track_ID;
			QString Track_Name;
			QString Track_Type;
			QString Track_Suffix;
			QString Track_Language;
			int lang_pos1,lang_pos2;

			Track_ID.setNum(x);
			Track_Name=extractTableWidget->item(x,1)->text();
			Track_Type=extractTableWidget->item(x,2)->text();
			lang_pos1=Track_Name.indexOf("(");
			lang_pos2=Track_Name.indexOf("):");
			Track_Language=Track_Name.mid(lang_pos1+1,lang_pos2-lang_pos1-1);
			current_Track = Track_ID+":"+BaseFile+"-"+Track_Type;
			if(Track_Type=="Audio" || Track_Type=="Subtitles")
				current_Track+="-"+Track_Language+Track_ID;

			Track_Suffix=extractTableWidget->item(x,3)->text();
			if(Track_Suffix.isEmpty())
				current_Track+="."+Track_Type.mid(0,3).toLower();
			else
				current_Track+="."+Track_Suffix;

			if(current_CheckBox[x]->isChecked())
				{
				arg << current_Track;
				}
			}

		QString Befehl="mkvextract ";
		Befehl+=arg.join(" ");
		extractTextEdit->setText(Befehl);

		extractProc.start("mkvextract", arg);
		}
	else if(extractStartButton->text()=="Stop")
		{
		extractProc.terminate();
		}
}


// extraction beendet

void mkvextractgui::extractProcBeendet(int exitCode, QProcess::ExitStatus exitStatus)
{
	extractStartButton->setText("Extract");
	extractOpenButton->setEnabled(true);
	extractSaveButton->setEnabled(true);
	extractOpenButton->setFocus();

	if(exitStatus==QProcess::CrashExit)
		{
		extractTextEdit->setText("FEHLER: mkvextract wurde beendet!");
		extractStartButton->setEnabled(false);
		}
	else if(exitCode!=0)
		{
		extractTextEdit->setText("Extraktions-Vorgang wurde abgebrochen!!!");
		extractStartButton->setEnabled(false);
		}
	else	{
		extractStartButton->setEnabled(false);
		extractProgressBar->setValue(100);
		i=0;

		// Sound abspielen
		QString verzeichnis=QCoreApplication::applicationDirPath();
		verzeichnis.append("/extraktion_beendet.WAV");
		QStringList wavfile;
		wavfile.clear();
		wavfile << verzeichnis;
		soundplay.start("mplayer",wavfile);

		// Message-Box für Fertig
		QApplication::beep();
		QMessageBox::information(this, "mkvextract-gui Finished",
				tr("Extraktions-Vorgang abgeschlossen!"),
				QMessageBox::Ok);
		}
}



/********************************************************************************************/
/*       QProgessBar (Fortschrittsbalken) für extract Werte eintragen                      */
/********************************************************************************************/

void mkvextractgui::extractFortschritt()
{
	QByteArray data= extractProc.readAllStandardOutput();
	QString text= QString::fromLocal8Bit(data);

	QRegExp reg("Fortschritt: [0-9][0-9]{1,2}");
	if(text.contains(reg))
		{
		QString fortschritt=reg.cap();
		int end_mark;
		end_mark=fortschritt.indexOf("%");
		QString nummer=fortschritt.mid(13,13-end_mark);

		double proz_nummer=nummer.toDouble();
		extractProgressBar->setValue(proz_nummer);
		}
}


/*********************************************************/
/*          extractInfo Prozess starten                    */
/*********************************************************/

void mkvextractgui::extractInfoStart()
{
	QString SourceFile;
	SourceFile = extractOpenLineEdit->text();

	QString verzeichnis=QCoreApplication::applicationDirPath();
	verzeichnis.append("/mediainfo_output_format_INFO.txt");

	QString mediainfo_output_format = "--Inform=file:///"+verzeichnis;

	QStringList arg;
	arg << mediainfo_output_format;
	arg << SourceFile;
	extractInfo.start("mediainfo", arg);
}


/**************************************************************************************/
/*          Ausgabe von extractInfo an Variable fileProperties zuweisen                 */
/**************************************************************************************/

void mkvextractgui::extractInfoFortschritt()
{
	QByteArray data= extractInfo.readAllStandardOutput();
	QString text= QString::fromLocal8Bit(data);

	fileProperties=text;
	fileProperties.chop(2);
}


/*********************************************************/
/*   extractInfo beendet  CHECKBOXEN WERDEN ERSTELLT!    */
/*********************************************************/

void mkvextractgui::extractInfoBeendet()
{
	QStringList singleTracks;
	singleTracks = fileProperties.split("\n");

	QFont font;
	font.setPointSize(10);

	extractTableWidget->setRowCount(singleTracks.size());

	int x;
	for(x=0;x<singleTracks.size();x++)
		{
		// Checkbox - Spalte 0
		QString CheckBoxName;
		CheckBoxName.setNum(x);
		CheckBoxName = "TrackCheckBox"+CheckBoxName;

		QCheckBox *extractTracksCheckBox = new QCheckBox();
		extractTracksCheckBox->setObjectName(CheckBoxName);
		extractTableWidget->setCellWidget(x,0, extractTracksCheckBox);
		connect(extractTracksCheckBox, SIGNAL(stateChanged(int)), this, SLOT(extractCheckBoxchecked()));

		// Track - Spalte 1
		if(!singleTracks[x].isEmpty())
			{
			extractTableWidget->setItem(x,1, new QTableWidgetItem(singleTracks[x]));
			extractTableWidget->item(x,1)->setFont(font);
			}

		// Type - Spalte 2
		if(singleTracks[x].contains("Video"))
			{
			extractTableWidget->setItem(x,2, new QTableWidgetItem("Video"));
			extractTableWidget->item(x,2)->setFont(font);
			}
		else if(singleTracks[x].contains("Audio"))
			{
			extractTableWidget->setItem(x,2, new QTableWidgetItem("Audio"));
			extractTableWidget->item(x,2)->setFont(font);
			}
		else if(singleTracks[x].contains("Text"))
			{
			extractTableWidget->setItem(x,2, new QTableWidgetItem("Subtitles"));
			extractTableWidget->item(x,2)->setFont(font);
			}

		// Suffix - Spalte 3
		if(singleTracks[x].contains("AVC"))
			{
			extractTableWidget->setItem(x,3, new QTableWidgetItem("mp4"));
			extractTableWidget->item(x,3)->setFont(font);
			}
		else if(singleTracks[x].contains("VC-1"))
			{
			extractTableWidget->setItem(x,3, new QTableWidgetItem("vc1"));
			extractTableWidget->item(x,3)->setFont(font);
			}
		else if(singleTracks[x].contains("AC-3"))
			{
			extractTableWidget->setItem(x,3, new QTableWidgetItem("ac3"));
			extractTableWidget->item(x,3)->setFont(font);
			}
		else if(singleTracks[x].contains("AAC"))
			{
			extractTableWidget->setItem(x,3, new QTableWidgetItem("aac"));
			extractTableWidget->item(x,3)->setFont(font);
			}
		else if(singleTracks[x].contains("DTS"))
			{
			extractTableWidget->setItem(x,3, new QTableWidgetItem("dts"));
			extractTableWidget->item(x,3)->setFont(font);
			}
		else if(singleTracks[x].contains("Opus"))
			{
			extractTableWidget->setItem(x,3, new QTableWidgetItem("opus"));
			extractTableWidget->item(x,3)->setFont(font);
			}
		else if(singleTracks[x].contains("PGS") && singleTracks[x].contains("BDs/HD-DVDs"))
			{
			extractTableWidget->setItem(x,3, new QTableWidgetItem("sup"));
			extractTableWidget->item(x,3)->setFont(font);
			}
		else if(singleTracks[x].contains("VobSub") && singleTracks[x].contains("DVDs"))
			{
			extractTableWidget->setItem(x,3, new QTableWidgetItem("sub"));
			extractTableWidget->item(x,3)->setFont(font);
			}
		else if(singleTracks[x].contains("Plain Text"))
			{
			extractTableWidget->setItem(x,3, new QTableWidgetItem("srt"));
			extractTableWidget->item(x,3)->setFont(font);
			}
		else	{
			extractTableWidget->setItem(x,3, new QTableWidgetItem(""));
			extractTableWidget->item(x,3)->setFont(font);
			}
		}
}


/******************************************************************/
/*  An und Abschalten des Extract-Buttons durch die Checkboxen    */
/******************************************************************/

void mkvextractgui::extractCheckBoxchecked()
{
	QList<QCheckBox *> current_CheckBox = extractTableWidget->findChildren<QCheckBox *>();
	int x;
	int vorhanden=0;
	for(x=0;x<current_CheckBox.size();x++)
		{
		if(current_CheckBox[x]->isChecked())
			{
			vorhanden=1;
			}
		}

	if(vorhanden==1)
		extractStartButton->setEnabled(true);
	else
		extractStartButton->setEnabled(false);
}



/***************************************************************************************/
/* Test nach dem Starten ob mkvextract verfügbar, wenn nicht mit QMessageBox beenden  */
/*************************************************************************************/

void mkvextractgui::mkvextracttest()
{
	QFileInfo test("/usr/bin/mkvextract");
	if(!test.exists())
		{
		font.setPointSize(12);
		QMessageBox msgBox;
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.setFont(font);
		msgBox.setIcon(QMessageBox::Critical);
		msgBox.setWindowTitle("mkvextract-gui");
		msgBox.setText(tr("<b>Fehler: mkvextract nicht gefunden!</b><br><br>"
			"mkvextract-gui benötigt mkvextract aus dem Paket mkvtoolnix von bunkus.org.<br>"
			"Bitte installieren Sie das Paket mkvtoolnix."));
		msgBox.exec();
		exit(EXIT_FAILURE);
		}
}


/**********************************************************************************/
/* Test nach dem Starten ob mediainfo verfügbar, wenn nicht mit QMessageBox beenden  */
/**********************************************************************************/

void mkvextractgui::mediainfotest()
{
	QFileInfo test("/usr/bin/mediainfo");
	if(!test.exists())
		{
		font.setPointSize(12);
		QMessageBox msgBox;
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.setFont(font);
		msgBox.setIcon(QMessageBox::Critical);
		msgBox.setWindowTitle("mkvextract-gui");
		msgBox.setText(tr("<b>Fehler: mediainfo nicht gefunden!</b><br><br>"
			"mkvextract-gui benötigt mediainfo zum Erkennen<br>"
			"und klassifizieren der einzelnen mkv-Spuren.<br>"
			"Bitte installieren Sie das Paket mediainfo."));
		msgBox.exec();
		exit(EXIT_FAILURE);
		}
}



/****************************************************************************************/
/* Falls Eingabe und Ausgabe Filename derselbe sind - neuen mit Nummerierung erstellen  */
/****************************************************************************************/

QString mkvextractgui::createNewFilename(const QString &oldFileName)
{
	QString newFileName;
	QString suffix;
	QString number;
	int counting_number;
	int start_pos_digit;
	int end_pos_digit;

	newFileName=oldFileName;

	QRegExp regSuffix("\\....$");
	if(newFileName.contains(regSuffix))
		suffix=regSuffix.cap();

	QRegExp regNumber("-NEW[0-9]{1,}\\....$");
	if(newFileName.contains(regNumber))
		{
		start_pos_digit=newFileName.indexOf(regNumber)+4;
		end_pos_digit=newFileName.indexOf(suffix);
		number=newFileName.mid(start_pos_digit,end_pos_digit-start_pos_digit);

		if(!number.isEmpty())
			{
			counting_number=number.toInt();
			counting_number++;
			number.setNum(counting_number);
			newFileName.replace(start_pos_digit,end_pos_digit-start_pos_digit,QString(number));
			}
		}
	else	{
		QString newSuffix;
		newSuffix="-NEW1"+suffix;
		newFileName.replace(QString(suffix), QString(newSuffix));
		}
	return(newFileName);
}


/*********************************************************/
/*    Sortierung für File-Einträge in die QTableWidget   */
/*********************************************************/

 bool caseInsensitiveLessThan(const QString &s1, const QString &s2)
 {
     return s1.toLower() > s2.toLower();
 }


/*********************************************************/
/*        Member-Funktion öffnen Dialog                  */
/*********************************************************/

QStringList mkvextractgui::openDialog(QString initialDir,const char* Filter)
{
	QStringList InpFiles = QFileDialog::getOpenFileNames(
		this,
		"Open File(s)",
		initialDir,
		Filter);

	qSort(InpFiles.begin(),InpFiles.end(),caseInsensitiveLessThan);
	return InpFiles;
}


/*********************************************************/
/*        Member-Funktion speichern Dialog               */
/*********************************************************/

QString mkvextractgui::saveDialog(QString initialDir)
{
	QString SaveDir = QFileDialog::getExistingDirectory(
		this,
		tr("Save in Folder"),
		initialDir,
		QFileDialog::DontConfirmOverwrite);

	if(!SaveDir.endsWith("/"))
		SaveDir = SaveDir+"/";

	return SaveDir;
}

